#encoding: utf8

from math import sqrt, pi
import random

from .spaceshift import ModeRR, ModePOP
from .star import Star
from .capsule import Capsule
from .asteroid import Asteroid
from .shield import Shield
from .arrow import Arrow
from .keys import (LEFT, RIGHT, UP, DOWN, SHOOT, START)

class Shift:
    """Enum of modes"""
    REDREBEL, POP= range(2)



class Game:
    """Game class, core of the game
     """

    def __init__(self, modes, difficulty, width, height): 
        """Game constructor"""
        shift_classes = {Shift.REDREBEL: ModeRR, Shift.POP: ModePOP}
        self.modes = modes
        self.difficulty = difficulty
        self.health_capacity = 10000*(8./difficulty)
        self.health_current = self.health_capacity
        self.last_generation_count = 0
        self.width = width
        self.height = height
        self.list_of_stars = []
        self.list_of_capsules = []
        self.list_of_asteroids = []
        self.list_of_arrows = []
        self.list_of_shields = []
        self.players = [shift_classes[mode](difficulty, mode, self) for mode in modes]
        self.area_radius = max(width, height)
        self.health = 100
        self.timer = 20
        self.timer_timer = 60
        self.score = 0
        self.best_score = 0
        self.k_time = 0
        self.k_limit = 180
        self.bool_start = False
        self.bool_end = False
        
        self.generate_stars(self.area_radius)
        
        self.list_of_capsules.append(Capsule(-30, 30))
        self.list_of_asteroids.append(Asteroid(300, 300, -1, -2))
        self.list_of_arrows.append(Arrow())
        #print (self.list_of_capsules[0].x)
        
        #for player in self.players:
        #    print player.x
   
   
    def generate_stars(self, area_radius):
        """Create stars once for all"""
        density_random_factor = 5 #random.randint(0, 3)
        kr = density_random_factor
        number_little_stars = random.randint(2 * kr, max(20 * kr, 1))
        #print number_little_stars
        for k in range(number_little_stars):
            self.list_of_stars.append(Star(random.uniform(0, area_radius), random.uniform(0, area_radius)))
            
    def continuum_generate_stars(self, area_radius, keystate):
        """Create stars dynamicaly"""
        density_random_factor = 0 #random.randint(0, 3)
        kr = density_random_factor
        global_width = self.width * 3.
        number_little_stars = random.randint(0, 1) #random.randint(2 * kr, max(20 * kr, 1))
        for k in range(number_little_stars):
            #self.list_of_stars.append(Star(random.uniform(-global_width, global_width), random.uniform(area_radius, area_radius + 10), 0))
            
            pof = random.randint(0, 1)
            
            if LEFT in keystate and UP not in keystate and DOWN not in keystate:
                self.list_of_stars.append(Star(random.uniform(- 10, 0), random.uniform(-global_width, global_width)))
            if RIGHT in keystate and UP not in keystate and DOWN not in keystate:
                self.list_of_stars.append(Star(random.uniform(area_radius, area_radius + 10), random.uniform(-global_width, global_width)))
            if DOWN in keystate and LEFT not in keystate and RIGHT not in keystate:
                self.list_of_stars.append(Star(random.uniform(-global_width, global_width), random.uniform(- 10, 0)))
            if UP in keystate and LEFT not in keystate and RIGHT not in keystate:
                self.list_of_stars.append(Star(random.uniform(-global_width, global_width), random.uniform(area_radius, area_radius + 10)))
        

            if LEFT in keystate and DOWN in keystate and UP not in keystate and RIGHT not in keystate:
                if pof == 0:
                    self.list_of_stars.append(Star(random.uniform(- 10, 0), random.uniform(-global_width, global_width)))
                else:
                    self.list_of_stars.append(Star(random.uniform(-global_width, global_width), random.uniform(- 10, 0)))
                if RIGHT in keystate and DOWN in keystate and UP not in keystate and LEFT not in keystate:
                    if pof == 0:
                        self.list_of_stars.append(Star(random.uniform(area_radius, area_radius + 10), random.uniform(-global_width, global_width)))
                else:
                    self.list_of_stars.append(Star(random.uniform(-global_width, global_width), random.uniform(- 10, 0)))
            if LEFT in keystate and UP in keystate and DOWN not in keystate and RIGHT not in keystate:
                if pof == 0:
                    self.list_of_stars.append(Star(random.uniform(- 10, 0), random.uniform(-global_width, global_width)))
                else:
                    self.list_of_stars.append(Star(random.uniform(-global_width, global_width), random.uniform(area_radius, area_radius + 10)))
            if RIGHT in keystate and UP in keystate and DOWN not in keystate and LEFT not in keystate:
                if pof == 0:
                    self.list_of_stars.append(Star(random.uniform(area_radius, area_radius + 10), random.uniform(-global_width, global_width)))
                else:
                    self.list_of_stars.append(Star(random.uniform(-global_width, global_width), random.uniform(area_radius, area_radius + 10)))
           
            else:
                self.list_of_stars.append(Star(random.uniform(-global_width, global_width), random.uniform(area_radius, area_radius + 10)))
            
    def generate_asteroids(self, area_radius):
        """Create stars once for all"""
        number_asteroids = random.randint(75, 300)#(30, 150)
        #print number_little_stars
            
        for k in range(number_asteroids):
            parity_x = random.randint(0, 1)
            parity_y = random.randint(0, 1)
            parity_s_x = random.randint(0, 1)
            parity_s_y = random.randint(0, 1)
        
            speed_x = random.randint(1, 10)
            speed_y = random.randint(1, 10)
        
        
            if parity_x == 0:
                pr_x = -1
            if parity_x == 1:
                pr_x = 1
        
            if parity_y == 0:
                pr_y = -1
            if parity_y == 1:
                pr_y = 1
            
            if parity_s_x == 0:
                pr_s_x = -1
            if parity_s_x == 1:
                pr_s_x = 1
        
            if parity_s_y == 0:
                pr_s_y = -1
            if parity_s_y == 1:
                pr_s_y = 1
            
            self.list_of_asteroids.append(Asteroid(random.uniform(pr_x * area_radius, (1 + pr_x) * area_radius), random.uniform(pr_y * area_radius, (1 + pr_y) * area_radius), pr_s_x * speed_x, pr_s_y * speed_y))
            
    def collide_on_limit_stars(self, star):
        """For detect star which escaping from the screen"""
        if sqrt(star.x**2 + star.y**2) > self.area_radius * 1.2 :
            self.list_of_stars.remove(star)
              
    def collide_on_limit_asteroids(self, asteroid):
        """For detect asteroid which escaping from the screen"""
        if sqrt(asteroid.x**2 + asteroid.y**2) > self.area_radius * 4 :
            self.list_of_asteroids.remove(asteroid)
              
    def collide_capsule_and_player(self, capsule, player):
        """Bullet and CowUp collision"""
        hb_half_length = capsule.hit_box_length / 2. + player.radius
        if capsule.x - hb_half_length < player.x < capsule.x + hb_half_length and capsule.y - hb_half_length < player.y < capsule.y + hb_half_length:# and player.y < self.height:
            #self.score += (2 * bullet.damage * (500 + bullet.number_type**7))/100
            if capsule in self.list_of_capsules:
                new_width = 0
                new_height = 0
                
                kx = capsule.x
                ky = capsule.y
                
                while 0 <= new_width < self.width:
                    new_width = random.uniform((kx  - 3 * self.width), (kx  + 3 * self.width))
                while 0 <= new_height < self.height:
                    new_height = random.uniform((ky  - 3 * self.height), (ky  + 3 * self.height))
                self.list_of_capsules.remove(capsule)
                #self.list_of_capsules.append(Capsule(kx - 30, ky + 30))
                self.list_of_capsules.append(Capsule(new_width, new_height))
            
                self.score += 1
                self.timer += 3
                self.health = min (100, self.health + 2)
                player.shield_jauge = min (100, player.shield_jauge + 10)
    
    def circle_of_admission(self, shield, asteroid, player):
        """Bullet and CowUp collision"""
        hb_half_length = asteroid.radius / 2. + shield.radius
        if asteroid.x - hb_half_length < shield.x < asteroid.x + hb_half_length and asteroid.y - hb_half_length < shield.y < asteroid.y + hb_half_length:
            #self.score += (2 * bullet.damage * (500 + bullet.number_type**7))/100
            if asteroid in self.list_of_asteroids:
                self.list_of_asteroids.remove(asteroid)
                player.shield_jauge = max (0, player.shield_jauge - 10)
                
            if player.shield_jauge == 0:
                self.list_of_shields.remove(shield)
                
    def get_width(self):
        return self.width
    
    def update(self, keystates):
        """Main game loop component.
    """
        if self.bool_start == False and self.bool_end == False:
        
            for keystate in keystates:
                if START in keystate:
                    self.bool_start = True
    
        if self.bool_start == True:
            #for star, keystate in zip(self.list_of_stars, keystates):
            #    star.update(keystate)
            for player, keystate in zip(self.players, keystates):
                player.update(keystate)
            
                for capsule in self.list_of_capsules:
                    for arrow in self.list_of_arrows:
                        arrow.update(capsule, player, self.width, self.height)
            for star in self.list_of_stars:
                star.update(keystate)
            for capsule in self.list_of_capsules:
                capsule.update(keystate)
            for asteroid in self.list_of_asteroids:
                asteroid.update(keystate)
        
        
            if RIGHT in keystate or UP in keystate or DOWN in keystate or LEFT in keystate:
                self.continuum_generate_stars(self.area_radius, keystate)
            
                for star in self.list_of_stars:
                    self.collide_on_limit_stars(star)
        
            for player in self.players:
                for capsule in self.list_of_capsules:
                    self.collide_capsule_and_player(capsule, player)
        
            for asteroid in self.list_of_asteroids:
                self.collide_on_limit_asteroids(asteroid)
        
            for player in self.players:
                for asteroid in self.list_of_asteroids:
                    player.on_collide_asteroid(asteroid)
                
            for player in self.players:
                for asteroid in self.list_of_asteroids:
                    for shield in self.list_of_shields:
                        self.circle_of_admission(shield, asteroid, player)
                    
        

            #if self.health <= 0:#-100000 :
                #return False
           
            #else:
            if self.k_time < self.k_limit:
                self.k_time += 1
            else:
                self.generate_asteroids(self.area_radius)
                self.k_time = 0
                self.k_limit = random.randint(30, 300)
                #print len(self.list_of_asteroids)
            #return True
        
            if self.health <= 0:
                self.bool_start = False
                self.bool_end = True
            
        
            self.timer_timer -= 1
        
            if self.timer_timer <= 0:
                self.timer -= 1
                self.timer_timer = 60
            
            if self.timer <= 0:
                self.bool_start = False
                self.bool_end = True
    
    
        if self.bool_start == False and self.bool_end == True:
        
        
            #while (len(self.players) > 0):
            #        self.players.pop()
            while (len(self.list_of_stars) > 0):
                self.list_of_stars.pop()
            while (len(self.list_of_capsules) > 0):
                self.list_of_capsules.pop()
            while (len(self.list_of_asteroids) > 0):
                self.list_of_asteroids.pop()
            while (len(self.list_of_arrows) > 0):
                self.list_of_arrows.pop()
            while (len(self.list_of_shields) > 0):
                self.list_of_shields.pop()
                
            if self.score > self.best_score:
                self.best_score = self.score
        
            for keystate in keystates:
                if START in keystate:
                    self.score = 0
                    self.health = 100
                    self.timer = 20
                    self.bool_start = True
                    self.bool_end = False
                
                    #self.players = [shift_classes[mode](self.difficulty, mode, self) for mode in self.modes]
                    self.generate_stars(self.area_radius)
        
                    self.list_of_capsules.append(Capsule(-30, 30))
                    self.list_of_asteroids.append(Asteroid(300, 300, -1, -2))
                    self.list_of_arrows.append(Arrow())
            
        #if self.bool_end == True:
        
