from math import sqrt, pi, cos, sin

from .shield import Shield

from .keys import (LEFT, RIGHT, UP, DOWN, SHOOT)
#from .bullet import Bullet

class PositionType :
    "Enum of positions"""
    N, E, S, O, NE, SE, SO, NO = range(8)

class Spaceshift :
    """Spaceshift class definition defined by :
    _ its mode (a number)
    _ its game (a Game object)
    _ its position x (x, in pixels)
    _ its position y (y, in pixels)
    _ its magnet jauge recovery (in percents)
    _ its magnet jauge speed recovery (in percents)
    _ its magnet efficient area (in pixels)
    _ its hitbox length (in pixels)
    _ its speed (in rad by move)
    _ its circle location radius
    _ its score
    """

    def __init__(self,  difficulty, mode, game):
        """Spaceshift constructor"""
        self.game = game
        self.mode = mode
        self.score = 0
        self.hit_box_length = 10
        self.health_current = 10000
        self.radius = 45
        self.x = self.game.width / 2 #500
        self.y = self.game.height / 2 #500
        self.x_global_map = 0
        self.y_global_map = 0
        self.speed = 10
        self.position = PositionType.N
        self.half_square_length = 1000
        self.shield_jauge = 100
        self.bool_shield = False
        self.shield_countdown = 10
        self.shield_recover_countdown = 5

        self.angle = -pi/2.
        # Define speed
        frames_needed = (8+difficulty/4.)*60
        perimeter = 2*pi
        self.speed = perimeter / frames_needed


    #@property
    #def x(self):
        #return cos(self.angle) * self.radius


    #@property
    #def y(self):
        #return sin(self.angle) * self.radius



class ModeRR(Spaceshift) :
    """ModeRouge sub-class definition self-attributes:
    _ its half square length

    """

    def __init__(self, difficulty, mode, game):
        """ModeRouge constructor"""
        Spaceshift.__init__(self, difficulty, mode, game)
        # Define speed
        frames_needed = (8+difficulty/4.)*60
        perimeter = self.half_square_length*8
        self.speed = 1.75 * perimeter / frames_needed



    def update(self, keystate):
        """ModeRedRebel movement"""
        if LEFT in keystate and UP not in keystate and DOWN not in keystate:
			 #self.x = max(self.x - self.speed, 45)#-self.half_square_length + 300)
            self.position = PositionType.O
            self.x_global_map -= self.speed
        if RIGHT in keystate and UP not in keystate and DOWN not in keystate:
			#self.x = min(self.x + self.speed, self.half_square_length - 45)
            self.position = PositionType.E
            self.x_global_map += self.speed
        if DOWN in keystate and LEFT not in keystate and RIGHT not in keystate:
			#self.y = max(self.y - self.speed, 45)
            self.position = PositionType.S
            self.y_global_map -= self.speed
        if UP in keystate and LEFT not in keystate and RIGHT not in keystate:
			#self.y = min(self.y + self.speed, self.half_square_length - 45)#-self.half_square_length + 300)
            self.position = PositionType.N
            self.y_global_map += self.speed
			
	
        if LEFT in keystate and DOWN in keystate and UP not in keystate and RIGHT not in keystate:
			#self.x = max(self.x - self.speed, 45)#-self.half_square_length + 300)
			#self.y = max(self.y - self.speed, 45)#-self.half_square_length + 300)
            self.position = PositionType.SO
            self.x_global_map -= self.speed
            self.y_global_map -= self.speed
        if RIGHT in keystate and DOWN in keystate and UP not in keystate and LEFT not in keystate:
			#self.x = min(self.x + self.speed, self.half_square_length - 45)
			#self.y = max(self.y - self.speed, 45)#-self.half_square_length + 300)
            self.position = PositionType.SE
            self.x_global_map += self.speed
            self.y_global_map -= self.speed
        if LEFT in keystate and UP in keystate and DOWN not in keystate and RIGHT not in keystate:
			#self.x = max(self.x - self.speed, 45)#-self.half_square_length + 300)
			#self.y = min(self.y + self.speed, self.half_square_length - 45)#-self.half_square_length + 300)
            self.position = PositionType.NO
            self.x_global_map -= self.speed
            self.y_global_map += self.speed
        if RIGHT in keystate and UP in keystate and DOWN not in keystate and LEFT not in keystate:
			#self.x = min(self.x + self.speed, self.half_square_length - 45)
			#self.y = min(self.y + self.speed, self.half_square_length - 45)#-self.half_square_length + 300)
            self.position = PositionType.NE
            self.x_global_map += self.speed
            self.y_global_map += self.speed
			
	
        if SHOOT in keystate:
            if self.shield_jauge > 0 and self.bool_shield == False:
                #print len(self.game.list_of_bullets)
                self.game.list_of_shields.append(Shield(self.game.width, self.game.height))
                self.bool_shield = True
        
            if self.shield_jauge > 0 and self.bool_shield == True:
                self.shield_countdown = max (0, self.shield_countdown - 1)
        
            if self.shield_countdown == 0:
                self.shield_jauge = max(0, self.shield_jauge - 1)
                #print self.shield_jauge
                self.shield_countdown = 10
                    
            if self.shield_jauge == 0 and self.bool_shield == True:
                if (len(self.game.list_of_shields) > 0):
                    self.game.list_of_shields.pop()
                self.bool_shield = False
                self.shield_countdown = 10
        		
        if SHOOT not in keystate:
            if (len(self.game.list_of_shields) > 0):
                self.game.list_of_shields.pop()
            self.bool_shield = False
            self.shield_countdown = 10
        	
            if self.shield_recover_countdown > 0:
                self.shield_recover_countdown = max(0, self.shield_recover_countdown - 1)
        			
            if self.shield_recover_countdown == 0:
                self.shield_jauge = min(100, self.shield_jauge + 1)
                if self.shield_jauge < 100:
                    #print self.shield_jauge
                    self.shield_recover_countdown = 5
        			
        			
        	
    """if self.bool_cooldown == True:
        self.cooldown = self.cooldown + 1
    if self.cooldown >= 15:
        self.cooldown = 0
        self.bool_cooldown = False"""
		
    def on_collide_asteroid(self, asteroid):
        """Asteroid and SpaceShip collision"""
        hb_half_length = self.hit_box_length / 2. + asteroid.radius
        if self.x - hb_half_length < asteroid.x < self.x + hb_half_length and self.y - hb_half_length < asteroid.y < self.y + hb_half_length:
            #self.score += (2 * asteroid.damage * (500 + asteroid.number_type**7))/100
            self.game.list_of_asteroids.remove(asteroid)
            self.game.health -= 10
      	        
        
        
class ModePOP(Spaceshift) :
    """ModePOP sub-class definition self-attributes:
    _ its half square length

    """

    def __init__(self, difficulty, mode, game):
        """ModePOP constructor"""
        Spaceshift.__init__(self, difficulty, mode, game)
        self.x = 2
        self.y = 0
        
    def update(self, keystate):
        """ModPOP movement"""
        if LEFT in keystate:
            self.x = max(self.x - self.speed, -self.half_square_length)
        if RIGHT in keystate:
            self.x = min(self.x + self.speed, self.half_square_length)

