import math

import pyglet
from pyglet.gl import *

from spacesavior.game.keys import *


class GameRunner(pyglet.window.Window):
    def __init__(self, game, screen_size):
        width, height = screen_size
        pyglet.window.Window.__init__(self, width=width, height=height,
                                            caption='Space Savior',
                                            resizable=True)
        self.game = game

        self.keystates = [set(), set()]
        #TODO: keymap-independent bindings?
        self.keybindings = {
            # Player 1
            pyglet.window.key.LEFT: (0, LEFT),
            pyglet.window.key.RIGHT: (0, RIGHT),
            pyglet.window.key.UP: (0, UP),
            pyglet.window.key.DOWN: (0, DOWN),
            pyglet.window.key.SPACE: (0, SHOOT),
            pyglet.window.key.ENTER: (0, START)
        }

        ship_textures_n = pyglet.resource.image('red_manta_horned_2_n_t.png')
        ship_textures_e = pyglet.resource.image('red_manta_horned_2_e_t.png')
        ship_textures_s = pyglet.resource.image('red_manta_horned_2_s_t.png')
        ship_textures_o = pyglet.resource.image('red_manta_horned_2_o_t.png')
        ship_textures_ne = pyglet.resource.image('red_manta_horned_2_ne_t.png')
        ship_textures_se = pyglet.resource.image('red_manta_horned_2_se_t.png')
        ship_textures_so = pyglet.resource.image('red_manta_horned_2_so_t.png')
        ship_textures_no = pyglet.resource.image('red_manta_horned_2_no_t.png')
	
        star_textures = pyglet.resource.image('star.png')
        capsule_textures = pyglet.resource.image('capsule_2_t.png')
        asteroid_textures = pyglet.resource.image('asteroid_2_t.png')
        shield_textures = pyglet.resource.image('shield_t.png')
        select_screen_textures = pyglet.resource.image('screen_intro.png')
        
        arrow_textures_n = pyglet.resource.image('arrow_n_t.png')
        arrow_textures_e = pyglet.resource.image('arrow_e_t.png')
        arrow_textures_s = pyglet.resource.image('arrow_s_t.png')
        arrow_textures_o = pyglet.resource.image('arrow_o_t.png')
        arrow_textures_ne = pyglet.resource.image('arrow_ne_t.png')
        arrow_textures_se = pyglet.resource.image('arrow_se_t.png')
        arrow_textures_so = pyglet.resource.image('arrow_so_t.png')
        arrow_textures_no = pyglet.resource.image('arrow_no_t.png')
	
	
        self.ship_textures_n = [ship_textures_n.get_region(0, 0, 80, 80)]
        self.ship_textures_e = [ship_textures_e.get_region(0, 0, 80, 80)]
        self.ship_textures_s = [ship_textures_s.get_region(0, 0, 80, 80)]
        self.ship_textures_o = [ship_textures_o.get_region(0, 0, 80, 80)]
        self.ship_textures_ne = [ship_textures_ne.get_region(0, 0, 95, 95)]
        self.ship_textures_se = [ship_textures_se.get_region(0, 0, 95, 95)]
        self.ship_textures_so = [ship_textures_so.get_region(0, 0, 95, 95)]
        self.ship_textures_no = [ship_textures_no.get_region(0, 0, 95, 95)]
        
        self.star_textures = [star_textures.get_region(0, 0, 1, 1)]
        self.capsule_textures = [capsule_textures.get_region(0, 0, 50, 50)]
        self.asteroid_textures = [asteroid_textures.get_region(0, 0, 25, 25)]
        self.shield_textures = [shield_textures.get_region(0, 0, 150, 150)]
        self.select_screen_textures = [select_screen_textures.get_region(0, 0, 1000, 800)]
        
        self.arrow_textures_n = [arrow_textures_n.get_region(0, 0, 45, 45)]
        self.arrow_textures_e = [arrow_textures_e.get_region(0, 0, 45, 45)]
        self.arrow_textures_s = [arrow_textures_s.get_region(0, 0, 45, 45)]
        self.arrow_textures_o = [arrow_textures_o.get_region(0, 0, 45, 45)]
        self.arrow_textures_ne = [arrow_textures_ne.get_region(0, 0, 53, 53)]
        self.arrow_textures_se = [arrow_textures_se.get_region(0, 0, 53, 53)]
        self.arrow_textures_so = [arrow_textures_so.get_region(0, 0, 53, 53)]
        self.arrow_textures_no = [arrow_textures_no.get_region(0, 0, 53, 53)]
        
        # Fix anchors
        for img in self.ship_textures_n:
            img.anchor_x = img.width / 2.
            img.anchor_y = img.height / 2.
        for img in self.ship_textures_e:
            img.anchor_x = img.width / 2.
            img.anchor_y = img.height / 2.
        for img in self.ship_textures_s:
            img.anchor_x = img.width / 2.
            img.anchor_y = img.height / 2.
        for img in self.ship_textures_o:
            img.anchor_x = img.width / 2.
            img.anchor_y = img.height / 2.
        for img in self.ship_textures_ne:
            img.anchor_x = img.width / 2.
            img.anchor_y = img.height / 2.
        for img in self.ship_textures_se:
            img.anchor_x = img.width / 2.
            img.anchor_y = img.height / 2.
        for img in self.ship_textures_so:
            img.anchor_x = img.width / 2.
            img.anchor_y = img.height / 2.
        for img in self.ship_textures_no:
            img.anchor_x = img.width / 2.
            img.anchor_y = img.height / 2.
            
        
        for img in self.capsule_textures:
            img.anchor_x = img.width / 2.
            img.anchor_y = img.height / 2.
            
        for img in self.asteroid_textures:
            img.anchor_x = img.width / 2.
            img.anchor_y = img.height / 2.
            
        for img in self.shield_textures:
            img.anchor_x = img.width / 2.
            img.anchor_y = img.height / 2.
            
         #for img in self.select_screen_textures:
            #img.anchor_x = img.width / 2.
            #img.anchor_y = img.height / 2.
        
        
        for img in self.arrow_textures_n:
            img.anchor_x = img.width / 2.
            img.anchor_y = img.height / 2.
        for img in self.arrow_textures_e:
            img.anchor_x = img.width / 2.
            img.anchor_y = img.height / 2.
        for img in self.arrow_textures_s:
            img.anchor_x = img.width / 2.
            img.anchor_y = img.height / 2.
        for img in self.arrow_textures_o:
            img.anchor_x = img.width / 2.
            img.anchor_y = img.height / 2.
        for img in self.arrow_textures_ne:
            img.anchor_x = img.width / 2.
            img.anchor_y = img.height / 2.
        for img in self.arrow_textures_se:
            img.anchor_x = img.width / 2.
            img.anchor_y = img.height / 2.
        for img in self.arrow_textures_so:
            img.anchor_x = img.width / 2.
            img.anchor_y = img.height / 2.
        for img in self.arrow_textures_no:
            img.anchor_x = img.width / 2.
            img.anchor_y = img.height / 2.

        #self.fps_display = pyglet.clock.ClockDisplay()
                                
                             
    def on_key_press(self, symbol, modifiers):
        pyglet.window.Window.on_key_press(self, symbol, modifiers)
        if symbol in self.keybindings:
            player, key = self.keybindings[symbol]
            self.keystates[player].add(key)


    def on_key_release(self, symbol, modifiers):
        #pyglet.window.Window.on_key_release(self, symbol, modifiers)
        if symbol in self.keybindings:
            player, key = self.keybindings[symbol]
            self.keystates[player].discard(key)


    def update(self, dt):
        self.game.update(self.keystates)
        
        
    def on_draw(self):
        self.clear() #TODO

        background_batch = pyglet.graphics.Batch()
        batch = pyglet.graphics.Batch()
        sprites = []

        #TODO: various effects, white hole itself, ...
        
        if self.game.bool_start == False and self.game.bool_end == False:
            #Draw select screen
            sprite = pyglet.sprite.Sprite(self.select_screen_textures[0],
                x=0,
                y=0,
                batch=batch)
            #sprite.scale = star.radius/2.
            sprites.append(sprite)
            # Write
            label_title = pyglet.text.Label("SPACE SAVIOR",
                font_name='Times New Roman',
                font_size=100,
                x=30, 
                y=600)
                #anchor_x='center', anchor_y='center')
		        
            label_start = pyglet.text.Label("To start, press ENTER",
                font_name='Times New Roman',
                font_size=19,
                x=200, 
                y=100)
                #anchor_x='center', anchor_y='center')
		                  
	
        
        if self.game.bool_start == False and self.game.bool_end == True:
        
        # Write
            label_go = pyglet.text.Label("GAME OVER",
                font_name='Times New Roman',
                font_size=100,
                x=100, 
                y=600)
                #anchor_x='center', anchor_y='center')
		        
            label_start_again = pyglet.text.Label("To start again, press ENTER",
                font_name='Times New Roman',
                font_size=19,
                x=350, 
                y=250)
                #anchor_x='center', anchor_y='center')
		        
            score = "Score : " + str(self.game.score)
            label_score = pyglet.text.Label(score,
                font_name='Times New Roman',
                font_size=19,
                x=200, 
                y=200)
                #anchor_x='center', anchor_y='center')
		                  
            score = "Best Score : " + str(self.game.best_score)
            label_best_score = pyglet.text.Label(score,
                font_name='Times New Roman',
                font_size=19,
                x=200, 
                y=180)
                #anchor_x='center', anchor_y='center')
	
        if self.game.bool_start == True:

            # Draw stars
            for star in self.game.list_of_stars:
                sprite = pyglet.sprite.Sprite(self.star_textures[0],
                    x=star.x,
                    y=star.y,
                    batch=batch)
                #sprite.scale = star.radius/2.
                sprites.append(sprite)
		    
		
		
            # Draw shields
            for shield in self.game.list_of_shields:
                sprite = pyglet.sprite.Sprite(self.shield_textures[0],
                    x=shield.x,
                    y=shield.y,
                    batch=batch)
                #sprite.scale = star.radius/2.
                sprites.append(sprite)
		    
		   
            # Draw players
            for player in self.game.players:
                #print 'ert'
                if player.position == 0:#PositionType.N:
                    #print 'zer'
                    ship_type_texture = self.ship_textures_n[0]
                if player.position == 1:#PositionType.E:
                    #print 'zaf'
                    ship_type_texture = self.ship_textures_e[0]
                if player.position == 2:#PositionType.S:
                    ship_type_texture = self.ship_textures_s[0]
                if player.position == 3:#PositionType.O:
                    ship_type_texture = self.ship_textures_o[0]
                if player.position == 4:#PositionType.NE:
                    ship_type_texture = self.ship_textures_ne[0]
                if player.position == 5:#PositionType.SE:
                    ship_type_texture = self.ship_textures_se[0]
                if player.position == 6:#PositionType.SO:
                    ship_type_texture = self.ship_textures_so[0]
                if player.position == 7:#PositionType.NO:
                    ship_type_texture = self.ship_textures_no[0]
		      		
		      		
		      		
                '''background_sprite = pyglet.sprite.Sprite(self.ship_backgrounds[player.mode],
                        x=player.x, y=player.y,
                        batch=background_batch)'''
                sprite = pyglet.sprite.Sprite(ship_type_texture,
                    x=player.x,
                    y=player.y,
                    batch=batch)
                sprites.append(sprite)
		
	
		
            # Draw capsules
            for capsule in self.game.list_of_capsules:
                sprite = pyglet.sprite.Sprite(self.capsule_textures[0],
                    x=capsule.x,
                    y=capsule.y,
                    batch=batch)
                #sprite.scale = star.radius/2.
                sprites.append(sprite)
		    
		
            # Draw asteroids
            for asteroid in self.game.list_of_asteroids:
                sprite = pyglet.sprite.Sprite(self.asteroid_textures[0],
                    x=asteroid.x,
                    y=asteroid.y,
                    batch=batch)
                #sprite.scale = star.radius/2.
                sprites.append(sprite)
		
            # Draw arrows
            for arrow in self.game.list_of_arrows:
                #print 'ert'
                if arrow.position == 0:#PositionType.N:
                    #print 'zer'
                    arrow_type_texture = self.arrow_textures_n[0]
                if arrow.position == 1:#PositionType.E:
                    #print 'zaf'
                    arrow_type_texture = self.arrow_textures_e[0]
                if arrow.position == 2:#PositionType.S:
                    arrow_type_texture = self.arrow_textures_s[0]
                if arrow.position == 3:#PositionType.O:
                    arrow_type_texture = self.arrow_textures_o[0]
                if arrow.position == 4:#PositionType.NE:
                    arrow_type_texture = self.arrow_textures_ne[0]
                if arrow.position == 5:#PositionType.SE:
                    arrow_type_texture = self.arrow_textures_se[0]
                if arrow.position == 6:#PositionType.SO:
                    arrow_type_texture = self.arrow_textures_so[0]
                if arrow.position == 7:#PositionType.NO:
                    arrow_type_texture = self.arrow_textures_no[0]
		      		
		      		
		      		
                '''background_sprite = pyglet.sprite.Sprite(self.arrow_backgrounds[arrow.mode],
				                             x=arrow.x, y=arrow.y,
				                             batch=background_batch)'''
                if arrow.position != 8:#PositionType.VOID:
		
                    sprite = pyglet.sprite.Sprite(arrow_type_texture,
                        x=arrow.x,
                        y=arrow.y,
                        batch=batch)
                    sprites.append(sprite)
			
	
            # Write
            for player in self.game.players:
                shield_pourcent = "Shield :" + str(player.shield_jauge) + "%"
                label_shield = pyglet.text.Label(shield_pourcent,
                    font_name='Times New Roman',
                    font_size=19,
                    x=70, 
                    y=70,
                    anchor_x='center', anchor_y='center')
		        
                health_pourcent = "Health :" + str(self.game.health) + "%"
                label_health = pyglet.text.Label(health_pourcent,
                    font_name='Times New Roman',
                    font_size=19,
                    x=70, 
                    y=50,
                    anchor_x='center', anchor_y='center')
		                  
                label_time = pyglet.text.Label(str(self.game.timer),
                    font_name='Times New Roman',
                    font_size=50,
                    x=170, 
                    y=750,
                    anchor_x='center', anchor_y='center')

		    
		    
        # Draw all
        background_batch.draw()
        batch.draw()
            
        glMatrixMode(GL_MODELVIEW)
        glLoadIdentity()
        #self.fps_display.draw()
        
        if self.game.bool_start == False and self.game.bool_end == False:
            label_title.draw()
            label_start.draw()
        
        if self.game.bool_start == True:
            label_shield.draw()
            label_health.draw()
            label_time.draw()
		
	
        if self.game.bool_start == False and self.game.bool_end == True:
            label_go.draw()
            label_start_again.draw()
            label_score.draw()
            label_best_score.draw()
            
                
